--[[This file creates a WorldMap class 
	And all the functions associated with it: changing tiles, generating the arrays, querying locations, etc..
--]]

WorldMap = Core.class(Sprite)
 
function WorldMap:init()
--[[Logic:  create a table for all the layer keys (self.mapArrays) and for each TileMap (self.mapLayers)
	Calls generateTerrain, placeMonsters, returnTileMap
--]]
	self.mapArrays = {}
	self.mapLayers = {}
	
	--fill two self.mapArrays
	self.mapArrays[LAYER_TERRAIN] = self:generateTerrain()
	self.mapArrays[LAYER_MONSTERS] = self:placeMonsters()
	
	-- fill two self.mapLayers and add to a Sprite as well
	local group = Sprite.new()
	local layer = self:returnTileMap(self.mapArrays[LAYER_TERRAIN], "images/tileset-background-108px.png")
	group:addChild(layer) 
	table.insert(self.mapLayers, layer)
	layer = self:returnTileMap(self.mapArrays[LAYER_MONSTERS], "images/tileset-monsters-108px.png")
	group:addChild(layer) 
	table.insert(self.mapLayers, layer)

	--add the tilemaps to the stage
	self:addChild(group)
end

function WorldMap:generateTerrain()
--[[Logic:  return an array to represent the background tiles
	Returns a table of size LAYER_COLUMNS * LAYER_ROWS
--]]

	return {
		3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3,
		3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3}
end

function WorldMap:placeMonsters()
--[[Logic:  return an array to represent the monsters tiles
	Returns a table of size LAYER_COLUMNS * LAYER_ROWS
--]]

	return {
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
end

function WorldMap:returnTileMap(array, tileset)
--[[Logic:  return a TileMap for a given tileset	
	Returns a TileMap
--]]

	local texture = Texture.new(tileset, true, {transparentColor = 0xff00ff})
	local tileMap = TileMap.new(LAYER_COLUMNS, LAYER_ROWS, texture, TILE_WIDTH, TILE_HEIGHT)
	
	--assign create each tile to the TileMap 
	for y = 1, LAYER_ROWS do
		for x = 1, LAYER_COLUMNS do
			local tX = array[x + (y - 1) * LAYER_COLUMNS]
			local tY = 1
			tileMap:setTile(x, y, tX, tY)
		end
	end
	return tileMap
end

function WorldMap:getTileKey(x, y, layer) 
--[[Logic:  return the tile key and the layer for a location on the map. 
			if no TileMap layer is specified, it returns the highest layer with a value.  i.e. monster #2 or background #1
	Returns key, layer
--]]
	local index = x + (y - 1) * LAYER_COLUMNS
	if layer then
		local array = self.mapArrays[layer]
		return array[index], layer				--return the value for the given layer 
	else
		for n = #self.mapArrays, 1, -1 do		--go through the TileMaps highest layer first
			local array = self.mapArrays[n]
			if array[index] ~= 0 then			--find the first value that isn't 0
				return array[index], n			--return the value and layer it's on
			end
		end
	end
end

function WorldMap:shiftWorld(dx, dy)
--[[Logic:  shift all the TileMaps by dx and dy 
			This is tile-based movement
--]]

	for i = 1, #self.mapLayers do
		local layer = self.mapLayers[i]
		layer:setX(layer:getX() - (dx * TILE_WIDTH))	
		layer:setY(layer:getY() - (dy * TILE_HEIGHT))
	end
end
	
function WorldMap:moveHero(hero, dx, dy)
--[[Logic:  change the mapArrays and mapLayers location of the hero and shift the TileMaps
	Calls shiftWorld
	Changes hero.x, hero.y too
--]]

	--get the tile entry that represents the hero
	local array = self.mapArrays[LAYER_MONSTERS]
	
	--erase the hero in the mapArrays and the mapLayers TileMap
	array[hero.x + (hero.y - 1) * LAYER_COLUMNS] = 0
	self.mapLayers[LAYER_MONSTERS]:clearTile(hero.x, hero.y)
	
	--place the hero in the mapArrays and the mapLayers TileMap at the new position
	array[hero.x + dx + (hero.y + dy - 1) * LAYER_COLUMNS] = 1
	self.mapLayers[LAYER_MONSTERS]:setTile(hero.x + dx, hero.y + dy, 1, 1) 	

	--keep the hero centered on the screen by shifting all the TileMaps  
	self:shiftWorld(dx, dy)
	
	--update the hero's location
	hero.x = hero.x + dx
	hero.y = hero.y + dy	
end